 

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using ESRI.ArcGIS.Mobile;
using ESRI.ArcGIS.Mobile.Gps;
using ESRI.ArcGIS.Mobile.MobileServices;

namespace WinGPS
{
  public partial class Form1 : Form
  {
    GpsConnection GpsConn = null;
    GpsDisplay gpsDisplay1 = new GpsDisplay();
    // This delegate enables asynchronous calls for setting
    // the text property on a TextBox control.
    delegate void SetTextCallback(string text);

    public Form1()
    {
      InitializeComponent();
    }
    private void Form1_Load(object sender, EventArgs e)
      {

        butDiscon.Enabled = false;
        //serialPortGpsConnection1.PortName = "COM5"; // can be hardcoded
        // lblGPSComPort.Text = serialPortGpsConnection1.PortName;
        // Sets the mobile cache directory to folder in MyDocuments if you uncomment this line
        //Use the Mobile Cache in the sample folder
        //mobileServerConnection1.Url = @"http://<yourserver.domain.com>/arcgis/services/<YOURSERVICE>/mapserver/mobileserver";
        mobileCache1.StoragePath = System.IO.Directory.GetParent(System.IO.Directory.GetParent(System.IO.Directory.GetParent(System.IO.Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\MapCache";
        
        if (!mobileCache1.CacheExists)
        {
          MessageBox.Show("Specified MobileService Cache is Not valid");
          return;
        }
        //create the cache and open
        try
        {
            mobileCache1.Open();
        }
        catch (Exception ex)
        {
          MessageBox.Show("Exception: " + ex.ToString());
        }
      
    }
    private void butCon_Click(object sender, EventArgs e)
    {
      butCon.Enabled = false;
      butDiscon.Enabled = true;
      //Use fileGPSConnection if you are reading GPS data from a file
      fileGpsConnection1.Enabled = true;
      fileGpsConnection1.FileName = System.IO.Directory.GetParent(System.IO.Directory.GetParent(System.IO.Directory.GetCurrentDirectory()).ToString()).ToString() + "\\GPS.txt";
      GpsConn = fileGpsConnection1;
      //Use serialportGPSConnection if you are reading GPS data from a device
      //   serialPortGpsConnection1.Open();
      //   GpsConn = serialPortGpsConnection1;
      GpsConn.Open();
      gpsDisplay1.GpsConnection = GpsConn;
      gpsDisplay1.Map = map1;
      if (GpsConn.IsOpen)
      {
        timer1.Enabled = true;
      }
      else
      {
        butCon.Enabled = true;
        butDiscon.Enabled = false;
        timer1.Enabled = false;
      }
    }
    private void butDiscon_Click(object sender, EventArgs e)
    {
      butDiscon.Enabled = false;
      butCon.Enabled = true;
      timer1.Enabled = false;
      GpsConn.Close();
    }
    private void timer1_Tick(object sender, EventArgs e)
    {
      DisplayGPSInfo(GpsConn);
      DisplaySatellites(GpsConn);
     
    }
    private void map1_ExtentChanged(object sender, EventArgs e)
    {
    //If a server is setup and a mobile service published it can be used to get addditional data
    }
    private void DisplayGPSInfo(GpsConnection GPSConn)
    {

      if (GPSConn == null)
          return;

      labelLatitude.Text = GPSConn.Latitude.ToString();
      labelLongitude.Text = GPSConn.Longitude.ToString();
      labelAltitude.Text = GPSConn.Altitude.ToString();

      labelFixMode.Text = GPSConn.FixStatus.ToString();

      labelPDOP.Text = GPSConn.PositionDilutionOfPrecision.ToString();
      labelVDOP.Text = GPSConn.VerticalDilutionOfPrecision.ToString();
      labelHDOP.Text = GPSConn.HorizontalDilutionOfPrecision.ToString();

      DateTime utc = DateTime.MinValue;
      utc = GPSConn.DateTime;

      labelDate.Text = utc.ToShortDateString();
      labelTimeLocal.Text = utc.ToLocalTime().ToString();
      labelTime.Text = utc.ToShortTimeString();
    }
       
    private void butClear_Click(object sender, EventArgs e)
    {
      NMEAText.Clear();
    }
    private void DisplaySatellites(GpsConnection GPSConn)
    {
      labelSatellitesInView.Text = GPSConn.FixSatelliteCount.ToString();

			//code from Coding4Fun: http://msdn.microsoft.com/coding4fun/someassemblyrequired/whereami/default.aspx

			Pen circlePen = new Pen(System.Drawing.Color.DarkBlue,1);

			Graphics g = picSats.CreateGraphics();
		
			int centerX = picSats.Width/2;
			int centerY = picSats.Height/2;

			double maxRadius = (Math.Min(picSats.Height,picSats.Width)-20) / 2;

			//draw circles
			double[] elevations = new double[] {0,Math.PI/2, Math.PI/3 ,Math.PI / 6};

			foreach(double elevation in elevations)
			{
				double radius = (double)System.Math.Cos(elevation) * maxRadius;
				g.DrawEllipse(circlePen,(int)(centerX - radius) ,(int)(centerY - radius),(int)(2 * radius),(int)( 2*  radius));
			}
			//90 degrees elevation reticule
			g.DrawLine(circlePen,new Point(centerX-3,centerY),new Point(centerX + 3,centerY));
			g.DrawLine(circlePen,new Point(centerX,centerY-3),new Point(centerX,centerY+3));

      Pen satPen = new Pen(System.Drawing.Color.Green, 4);

      listSatellites.Items.Clear();

      try
      {
        foreach (Satellite sat in GPSConn.Satellites)
        {
          if (null == sat) return;
          ListViewItem lvItem = new ListViewItem
            (
              new string[] 
              {
                sat.Id.ToString(),
                sat.Elevation.ToString(),
                sat.Azimuth.ToString(),
								sat.SignalStrength.ToString(),
              }
            );

          listSatellites.Items.Add(lvItem);

          //draw satellites
          double h = (double)System.Math.Cos((sat.Elevation * Math.PI) / 180) * maxRadius;

          int satX = (int)(centerX + h * Math.Sin((sat.Azimuth * Math.PI) / 180));
          int satY = (int)(centerY - h * Math.Cos((sat.Azimuth * Math.PI) / 180));

          g.DrawRectangle(satPen, satX, satY, 4, 4);
          g.DrawString(sat.Id.ToString(), new Font("Verdana", 8, FontStyle.Regular), new System.Drawing.SolidBrush(Color.Black), new Point(satX + 5, satY + 5));
        }
      }
      catch
      {
        return;
      }
    }

    private void fileGpsConnection1_SentenceReceived(object sender, NmeaSentenceEventArgs e)
    {
       if (chkRaw.Checked)
       {
         this.SetText(e.Sentence);
       }
    }
     
    private void SetText(string text)
    {
      // InvokeRequired required compares the thread ID of the
      // calling thread to the thread ID of the creating thread.
      // If these threads are different, it returns true.
      if (this.NMEAText.InvokeRequired)
      {
        SetTextCallback d = new SetTextCallback(SetText);
        this.Invoke(d, new object[] { text });
      }
      else
      {
        this.NMEAText.Text = NMEAText.Text + text + "\r\n";
      }
    }
   
  }
}

